<?php

namespace Drupal\subentity\Generators;

use DrupalCodeGenerator\Command\BaseGenerator;
use DrupalCodeGenerator\Utils;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\Question;
use Symfony\Component\Console\Question\ConfirmationQuestion;

/**
 * SubEntityGenerator class.
 */
class SubEntityGenerator extends BaseGenerator {

  /**
   * Name variable.
   *
   * @var string
   */
  protected $name = 'subentity';

  /**
   * Description variable.
   *
   * @var string
   */
  protected $description = 'Generates a subentity.';

  /**
   * Template path variable.
   *
   * @var string
   */
  protected $templatePath = __DIR__ . '/../../templates/';

  /**
   * {@inheritDoc}
   */
  public function __construct($name = NULL) {
    parent::__construct($name);
  }

  /**
   * {@inheritdoc}
   */
  protected function interact(InputInterface $input, OutputInterface $output) {
    $questions = Utils::moduleQuestions();
    $questions['label'] = new Question('Entity label');
    $questions['entity_class'] = new Question('Class name');
    $questions['bundle'] = new ConfirmationQuestion('Has bundle', FALSE);

    $vars = &$this->collectVars($input, $output, $questions);
    $vars['entity_name'] = $this->camelToSnakeCase($vars['entity_class']);
    $vars['module'] = $vars['machine_name'];

    $this->addFile()
      ->path('src/Entity/{entity_class}.php')
      ->template('generator/subentity.php.twig');

    $this->addFile()
      ->path('{machine_name}.links.menu.yml')
      ->template('generator/links.menu.yml.twig')
      ->action('append');

    $this->addFile()
      ->path('{machine_name}.links.task.yml')
      ->template('generator/links.task.yml.twig')
      ->action('append');

    $this->addFile()
      ->path('{machine_name}.routing.yml')
      ->template('generator/routing.yml.twig')
      ->action('append');

    if ($vars['bundle']) {
      $this->addFile()
        ->path('src/Entity/{entity_class}Type.php')
        ->template('generator/subentity_type.php.twig');

      $this->addFile()
        ->path('config/schema/{entity_name}_type.yml')
        ->template('generator/subentity_type.schema.yml');

      $this->addFile()
        ->path('{machine_name}.links.action.yml')
        ->template('generator/links.action.yml.twig');
    }

  }

  /**
   * Convert camel case to snake case.
   *
   * @param string $string
   *   String to convert.
   *
   * @return string
   *   Converted string.
   */
  protected function camelToSnakeCase($string) {
    return strtolower(preg_replace('/[A-Z]/', '_\\0', lcfirst($string)));
  }

}
